package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.context.support.ResourceBundleMessageSource;
import org.springframework.validation.BindingResult;
import org.springframework.validation.DataBinder;
import org.springframework.validation.ObjectError;
import org.springframework.validation.Validator;
import jakarta.validation.ConstraintViolation;
import java.awt.print.Book;
import java.util.Comparator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

public class GenericValidator {

    @Autowired
    ApplicationContext context;

    /**
     * This is a framework style generic method which can
     * validate any object given that the corresponding
     * validator is register as bean.
     */
    public boolean validateObject(Object objectToValidate) {

        Map<String, Validator> validatorMap = context.getBeansOfType(Validator.class);
        if (validatorMap == null) {
            return true;
        }

        DataBinder binder = new DataBinder(objectToValidate);

        //in this example two validators are register OrderValidator
        // and LocalValidatorFactoryBean which will do JSR 349 validations.
        for (Validator validator : validatorMap.values()) {

            if (validator.supports(objectToValidate.getClass())) {
                binder.addValidators(validator);
            }
        }

        binder.validate();
        BindingResult bindingResult = binder.getBindingResult();
        if (bindingResult.hasErrors()) {
            ResourceBundleMessageSource messageSource = new ResourceBundleMessageSource();
            messageSource.setBasename("ValidationMessages");

            System.err.println(messageSource.getMessage("object.invalid",
                    new Object[]{objectToValidate.getClass().getSimpleName()}, Locale.US));

            bindingResult.getAllErrors()
                         .stream()
                         .map(e -> messageSource.getMessage(e, Locale.US))
                         .sorted(Comparator.naturalOrder())
                         .forEach(System.err::println);
            return false;
        }

        return true;
    }
}